@@ -14,7 +14,7 @@ import paho.mqtt.client
 from PIL import Image
 import ascii_magic
 
-from weconnect import weconnect, addressable, errors, util
+from weconnect import weconnect, addressable, errors, util, domain
 from weconnect.__version import __version__ as __weconnect_version__
 
 from .__version import __version__
@@ -51,63 +51,72 @@ class NumberRangeArgument:
         return argparse.ArgumentTypeError('Must be a number')
 
 
-def main():  # noqa: C901  # pylint: disable=too-many-branches,too-many-statements
+def main():  # noqa: C901  # pylint: disable=too-many-branches,too-many-statements,too-many-locals
     parser = argparse.ArgumentParser(
         prog='weconnect-mqtt',
         description='Commandline Interface to interact with the Volkswagen WeConnect Services')
     parser.add_argument('--version', action='version',
                         version=f'%(prog)s {__version__} (using WeConnect-python {__weconnect_version__})')
-    parser.add_argument('--mqttbroker', type=str, help='Address of MQTT Broker to connect to', required=True)
-    parser.add_argument('--mqttport', type=NumberRangeArgument(1, 65535), help='Port of MQTT Broker. Default is 1883 (8883 for TLS)',
-                        required=False, default=None)
-    parser.add_argument('--mqttclientid', required=False, default=None, help='Id of the client. Default is a random id')
-    parser.add_argument('-k', '--mqttkeepalive', required=False, type=int, default=60,
-                        help='Time between keep-alive messages')
-    parser.add_argument('-mu', '--mqtt-username', type=str, dest='mqttusername',
-                        help='Username for MQTT broker', required=False)
-    parser.add_argument('-mp', '--mqtt-password', type=str, dest='mqttpassword',
-                        help='Password for MQTT broker', required=False)
-    parser.add_argument('--transport', required=False, default='tcp', choices=["tcp", 'websockets'],
-                        help='EXPERIMENTAL support for websockets transport')
-    parser.add_argument('-s', '--use-tls', action='store_true', help='EXPERIMENTAL')
-    parser.add_argument('--insecure', action='store_true', help='EXPERIMENTAL')
-    parser.add_argument('--cacerts', required=False, default=None, help='EXPERIMENTAL path to the Certificate Authority'
-                        ' certificate files that are to be treated as trusted by this client')
-    parser.add_argument('--cert', required=False, default=None, help='EXPERIMENTAL PEM encoded client certificate')
-    parser.add_argument('--key', required=False, default=None, help='EXPERIMENTAL PEM encoded client private key')
-    parser.add_argument('--tls-version', required=False, default=None, choices=['tlsv1.2', 'tlsv1.1', 'tlsv1'],
-                        help='EXPERIMENTAL TLS protocol version')
+    brokerGroup = parser.add_argument_group('MQTT and Broker')
+    brokerGroup.add_argument('--mqttbroker', type=str, help='Address of MQTT Broker to connect to', required=True)
+    brokerGroup.add_argument('--mqttport', type=NumberRangeArgument(1, 65535), help='Port of MQTT Broker. Default is 1883 (8883 for TLS)',
+                             required=False, default=None)
+    brokerGroup.add_argument('--mqttclientid', required=False, default=None, help='Id of the client. Default is a random id')
+    brokerGroup.add_argument('--prefix', help='MQTT topic prefix', type=str, required=False, default='weconnect/0')
+    brokerGroup.add_argument('-k', '--mqttkeepalive', required=False, type=int, default=60, help='Time between keep-alive messages')
+    brokerGroup.add_argument('-mu', '--mqtt-username', type=str, dest='mqttusername', help='Username for MQTT broker', required=False)
+    brokerGroup.add_argument('-mp', '--mqtt-password', type=str, dest='mqttpassword', help='Password for MQTT broker', required=False)
+    brokerGroup.add_argument('--transport', required=False, default='tcp', choices=["tcp", 'websockets'], help='EXPERIMENTAL support for websockets transport')
+    brokerGroup.add_argument('-s', '--use-tls', action='store_true', help='EXPERIMENTAL')
+    brokerGroup.add_argument('--insecure', action='store_true', help='EXPERIMENTAL')
+    brokerGroup.add_argument('--cacerts', required=False, default=None, help='EXPERIMENTAL path to the Certificate Authority'
+                             ' certificate files that are to be treated as trusted by this client')
+    brokerGroup.add_argument('--cert', required=False, default=None, help='EXPERIMENTAL PEM encoded client certificate')
+    brokerGroup.add_argument('--key', required=False, default=None, help='EXPERIMENTAL PEM encoded client private key')
+    brokerGroup.add_argument('--tls-version', required=False, default=None, choices=['tlsv1.2', 'tlsv1.1', 'tlsv1'],
+                             help='EXPERIMENTAL TLS protocol version')
+    brokerGroup.add_argument('--ignore-for', dest='ignore', help='Ignore messages for first IGNORE seconds after subscribe to aviod '
+                             'retained messages from the broker to make changes to the car (default is 5s) if you don\'t want this behavious set to 0',
+                             type=int, required=False, default=5)
 
-    parser.add_argument('-u', '--username', type=str, help='Username of Volkswagen id', required=False)
-    parser.add_argument('-p', '--password', type=str, help='Password of Volkswagen id', required=False)
+    weConnectGroup = parser.add_argument_group('WeConnect')
+    weConnectGroup.add_argument('-u', '--username', type=str, help='Username of Volkswagen id', required=False)
+    weConnectGroup.add_argument('-p', '--password', type=str, help='Password of Volkswagen id', required=False)
     defaultNetRc = os.path.join(os.path.expanduser("~"), ".netrc")
-    parser.add_argument('--netrc', help=f'File in netrc syntax providing login (default: {defaultNetRc}).'
-                        ' Netrc is only used when username and password are not provided  as arguments',
-                        default=None, required=False)
-    parser.add_argument('-i', '--interval', help='Query interval in seconds',
-                              type=NumberRangeArgument(300, 3500), required=False, default=300)
-    parser.add_argument('--picture-cache-interval', dest='pictureCache', help='Picture download interval in seconds, this does not influence the interval in'
-                        ' which the status picture is updated', type=NumberRangeArgument(1), required=False, default=86400)
-    parser.add_argument('--prefix', help='MQTT topic prefix',
-                        type=str, required=False, default='weconnect/0')
-    parser.add_argument('--ignore-for', dest='ignore', help='Ignore messages for first IGNORE seconds after subscribe to aviod '
-                        'retained messages from the broker to make changes to the car (default is 5s) if you don\'t want this behavious set to 0',
-                        type=int, required=False, default=5)
-    parser.add_argument('-v', '--verbose', action="append_const", const=-1,)
-    parser.add_argument('-l', '--chargingLocation', nargs=2, metavar=('latitude', 'longitude'), type=float,
-                        help='If set charging locations will be added to the result around the given coordinates')
-    parser.add_argument('--chargingLocationRadius', type=NumberRangeArgument(0, 100000),
-                        help='Radius in meters around the chargingLocation to search for chargers')
-    parser.add_argument('--no-capabilities', dest='noCapabilities', help='Do not add capabilities', action='store_true')
+    weConnectGroup.add_argument('--netrc', help=f'File in netrc syntax providing login (default: {defaultNetRc}).'
+                                ' Netrc is only used when username and password are not provided  as arguments', default=None, required=False)
+    weConnectGroup.add_argument('-i', '--interval', help='Query interval in seconds',
+                                type=NumberRangeArgument(300, 3500), required=False, default=300)
+    weConnectGroup.add_argument('--picture-cache-interval', dest='pictureCache', help='Picture download interval in seconds, this does not influence the'
+                                ' interval in which the status picture is updated', type=NumberRangeArgument(1), required=False, default=86400)
+
+    weConnectGroup.add_argument('-l', '--chargingLocation', nargs=2, metavar=('latitude', 'longitude'), type=float,
+                                help='If set charging locations will be added to the result around the given coordinates')
+    weConnectGroup.add_argument('--chargingLocationRadius', type=NumberRangeArgument(0, 100000),
+                                help='Radius in meters around the chargingLocation to search for chargers')
+    weConnectGroup.add_argument('--no-capabilities', dest='noCapabilities', help='Do not add capabilities', action='store_true')
+    weConnectGroup.add_argument('--selective', help='Just fetch status of a certain type', default=None, required=False, action='append',
+                                type=domain.Domain, choices=list(domain.Domain))
     parser.add_argument('--pictures', help='Add ASCII art pictures', action='store_true')
 
+    loggingGroup = parser.add_argument_group('Logging')
+    loggingGroup.add_argument('-v', '--verbose', action="append_const", help='Logging level (verbosity)', const=-1,)
+    loggingGroup.add_argument('--logging-format', dest='loggingFormat', help='Logging format configured for python logging '
+                              '(default: %%(asctime)s:%%(levelname)s:%%(module)s:%%(message)s)', default='%(asctime)s:%(levelname)s:%(module)s:%(message)s')
+    loggingGroup.add_argument('--logging-date-format', dest='loggingDateFormat', help='Logging format configured for python logging '
+                              '(default: %%Y-%%m-%%dT%%H:%%M:%%S%%z)', default='%Y-%m-%dT%H:%M:%S%z')
+    loggingGroup.add_argument('--hide-repeated-log', dest='hideRepeatedLog', help='Hide repeated log messages from the same module', action='store_true')
+
     args = parser.parse_args()
 
     logLevel = LOG_LEVELS.index(DEFAULT_LOG_LEVEL)
     for adjustment in args.verbose or ():
         logLevel = min(len(LOG_LEVELS) - 1, max(logLevel + adjustment, 0))
 
-    logging.basicConfig(level=LOG_LEVELS[logLevel])
+    logging.basicConfig(level=LOG_LEVELS[logLevel], format=args.loggingFormat, datefmt=args.loggingDateFormat)
+    if args.hideRepeatedLog:
+        for handler in logging.root.handlers:
+            handler.addFilter(util.DuplicateFilter())
     LOG.info('WeConnect-mqtt %s (using WeConnect-python %s)', __version__, __weconnect_version__)
 
     usetls = args.use_tls
@@ -175,7 +184,7 @@ def main():  # noqa: C901  # pylint: disable=too-many-branches,too-many-statemen
 
     mqttCLient = WeConnectMQTTClient(clientId=args.mqttclientid, transport=args.transport, interval=args.interval,
                                      prefix=args.prefix, ignore=args.ignore, updateCapabilities=(not args.noCapabilities),
-                                     updatePictures=args.pictures)
+                                     updatePictures=args.pictures, selective=args.selective)
     mqttCLient.enable_logger()
 
     if usetls:
@@ -246,7 +255,8 @@ def main():  # noqa: C901  # pylint: disable=too-many-branches,too-many-statemen
 
 
 class WeConnectMQTTClient(paho.mqtt.client.Client):  # pylint: disable=too-many-instance-attributes
-    def __init__(self, clientId=None, transport='tcp', interval=300, prefix='weconnect/0', ignore=0, updateCapabilities=True, updatePictures=True):
+    def __init__(self, clientId=None, transport='tcp', interval=300, prefix='weconnect/0', ignore=0, updateCapabilities=True, updatePictures=True,
+                 selective=None):
         super().__init__(client_id=clientId, transport=transport)
         self.weConnect = None
         self.prefix = prefix
@@ -257,6 +267,7 @@ class WeConnectMQTTClient(paho.mqtt.client.Client):  # pylint: disable=too-many-
         self.lastSubscribe = None
         self.updateCapabilities = updateCapabilities
         self.updatePictures = updatePictures
+        self.selective = selective
 
         self.on_connect = self.on_connect_callback
         self.on_message = self.on_message_callback
@@ -274,7 +285,8 @@ class WeConnectMQTTClient(paho.mqtt.client.Client):  # pylint: disable=too-many-
     def connectWeConnect(self, username, password, maxAgePictures):
         LOG.info('Connect to WeConnect')
         self.weConnect = weconnect.WeConnect(username=username, password=password, updateAfterLogin=False, updateCapabilities=self.updateCapabilities,
-                                             updatePictures=self.updatePictures, maxAgePictures=maxAgePictures)
+                                             updatePictures=self.updatePictures, maxAgePictures=maxAgePictures, selective=self.selective)
+        self.weConnect.enableTracker()
         self.weConnect.addObserver(self.onWeConnectEvent, addressable.AddressableLeaf.ObserverEvent.VALUE_CHANGED
                                    | addressable.AddressableLeaf.ObserverEvent.ENABLED | addressable.AddressableLeaf.ObserverEvent.DISABLED,
                                    priority=addressable.AddressableLeaf.ObserverPriority.USER_MID)
@@ -284,7 +296,7 @@ class WeConnectMQTTClient(paho.mqtt.client.Client):  # pylint: disable=too-many-
     def updateWeConnect(self):
         LOG.info('Update data from WeConnect')
         try:
-            self.weConnect.update(updateCapabilities=self.updateCapabilities, updatePictures=self.updatePictures)
+            self.weConnect.update(updateCapabilities=self.updateCapabilities, updatePictures=self.updatePictures, selective=self.selective)
             self.setConnected(connected=True)
             self.setError(code=WeConnectErrors.SUCCESS)
             self.publish(topic=f'{self.prefix}/mqtt/weconnectUpdated', qos=1, retain=True,
